//
//  GMPullToAction.m
//  TestPullToRefresh
//
//  Created by 杨淳引 on 15/7/31.
//  Copyright (c) 2015年 杨淳引. All rights reserved.
//

#import "GMPullToAction.h"
#import "CircleProgessView.h"
#import "GMActivityView.h"

#define kFrameHeight 60
#define kTextColor [UIColor darkGrayColor]
#define kContent_Width    ([UIScreen mainScreen].applicationFrame.size.width)

enum {
    GMPullToRefreshStateHidden = 1,  //隐藏
    GMPullToRefreshStateVisible,     //可见
    GMPullToRefreshStateTriggered,   //已触发刷新
    GMPullToRefreshStateLoading      //已在加载
};

typedef NSUInteger GMPullToRefreshState;

@interface GMPullToRefresh ()
@property (nonatomic, copy) void (^actionHandler)(void);
@property (nonatomic, weak) UIScrollView *scrollView;
@property (nonatomic, strong) UILabel *titleLabel;
@property (nonatomic, strong) GMActivityView *activityView;
@property (nonatomic,strong)CircleProgessView *circleView;
@property (nonatomic, readwrite) GMPullToRefreshState state;
@property (nonatomic, strong, readonly) UIImage *arrowImage;
@end

@implementation GMPullToRefresh
- (id)initWithScrollView:(UIScrollView *)scrollView {
    //初始化
    self = [super initWithFrame:CGRectZero];
    self.scrollView = scrollView;
    self.frame = CGRectMake(0, -kFrameHeight, scrollView.bounds.size.width, kFrameHeight);
    [_scrollView addSubview:self];
    
    //定制提示文字
    self.titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(kContent_Width/2.f-75/4.f, self.bounds.size.height*0.5-10, 75, 20)];
    _titleLabel.font = [UIFont boldSystemFontOfSize:14];
    _titleLabel.backgroundColor = [UIColor clearColor];
    _titleLabel.textColor = kTextColor;
    [self addSubview:_titleLabel];
    
    //矩形上升动画图
    self.activityView=[self activityIndicatorView];
    
    //圆圈转动动画
    self.circleView=[[CircleProgessView alloc] initWithFrame:CGRectMake(self.titleLabel.frame.origin.x-30-5, self.bounds.size.height*0.5-15, 30, 30)];
    [self addSubview:self.circleView];
    
    //指定state，通过这个state可以判断文字和动画的显示效果
    self.state = GMPullToRefreshStateHidden;
    
    return self;
}

- (GMActivityView *)activityIndicatorView {
    if(!_activityView) {
        _activityView = [[GMActivityView alloc] initWithFrame:CGRectMake(self.titleLabel.frame.origin.x-30, self.bounds.size.height*0.5-10, 20, 20)];
        _activityView.hidesWhenStopped = NO;
        [self addSubview:_activityView];
    }
    return _activityView;
}

- (void)setState:(GMPullToRefreshState)newState {
    _state = newState;
    switch (newState) {
        case GMPullToRefreshStateHidden:
            [self.circleView stopAnimation];
            [self.activityView stopAnimation];
            self.activityView.isFull = NO;
            [self setScrollViewContentInsetTop:0];
            break;
            
        case GMPullToRefreshStateVisible:
            _titleLabel.text = NSLocalizedString(@"下拉刷新...",);
            [self.circleView stopAnimation];
            [self.activityView stopAnimation];
            self.activityView.isFull = NO;
            break;
            
        case GMPullToRefreshStateTriggered:
            _titleLabel.text = NSLocalizedString(@"松开刷新...",);
            self.circleView.transform=CGAffineTransformMakeRotation(0);
            self.activityView.isFull = YES;
            break;
            
        case GMPullToRefreshStateLoading:
            _titleLabel.text = NSLocalizedString(@"正在载入...",);
            self.circleView.transform=CGAffineTransformMakeRotation(0);
            [self.circleView startAnimation];
            [self.activityView startAnimation];
            [self setScrollViewContentInsetTop:self.frame.size.height];
            if(_actionHandler)
                _actionHandler();
            break;
    }
}

- (void)setScrollViewContentInsetTop:(CGFloat)top {
    UIEdgeInsets inset = self.scrollView.contentInset;
    inset.top = top;
    [UIView animateWithDuration:0.3 delay:0 options:UIViewAnimationOptionAllowUserInteraction|UIViewAnimationOptionBeginFromCurrentState animations:^{
        self.scrollView.contentInset = inset;
        
    } completion:^(BOOL finished) {
    }];
}

- (void)scrollViewDidScroll:(CGPoint)contentOffset {
    if (self.state == GMPullToRefreshStateLoading) {
        return;
    }
    
    //起点的y值（负数），它的绝对值也是pullToRefreshView的高度，也是触发刷新状态的高度
    CGFloat scrollOffsetThreshold = self.frame.origin.y;
    
    //已经触发刷新并且放开拖动了，变成加载状态
    if(self.state == GMPullToRefreshStateTriggered && !self.scrollView.isDragging){
        self.state = GMPullToRefreshStateLoading;
        
    //scrollView开始往下拖动（<0），并且未达到触发刷新的高度（>scrollOffsetThreshold），为可见状态
    }else if(contentOffset.y < 0 &&  contentOffset.y > scrollOffsetThreshold && self.scrollView.isDragging && self.state != GMPullToRefreshStateLoading){
        self.state = GMPullToRefreshStateVisible;
        
    //scrollView往下拖动到已触发刷新的高度（<scrollOffsetThreshold）并且还没放手，为触发状态
    }else if(contentOffset.y <= scrollOffsetThreshold && self.scrollView.isDragging && self.state == GMPullToRefreshStateVisible){
        self.state = GMPullToRefreshStateTriggered;
        
    //scrollView往上推，为隐藏状态
    }else if(contentOffset.y >= 0 && self.state != GMPullToRefreshStateHidden){
        self.state = GMPullToRefreshStateHidden;
    }
    
    if (self.state==GMPullToRefreshStateHidden||self.state==GMPullToRefreshStateVisible) {
        //取绝对值
        float moveY = fabs(self.scrollView.contentOffset.y);
        if (moveY > kFrameHeight)
            moveY = kFrameHeight;
        //根据拖动距离和触发距离的比例，来定制圆弧的长度
        [self.circleView drawCircleWithProgress:(moveY-kFrameHeight*0.5) / (kFrameHeight*0.5)];
        //用同样的方法来定制方形的高度
        [self.activityView drawViewWithProgress:(moveY/kFrameHeight)];
        
    } else if (self.state==GMPullToRefreshStateTriggered) {
        [self.circleView drawCircleWithProgress:1.];
        //超过触发距离了，圆圈继续转
        float moveY=fabs(self.scrollView.contentOffset.y);
        self.circleView.transform=CGAffineTransformMakeRotation((moveY-kFrameHeight)/60*360*M_PI/180);
    }
}

- (void)stopAnimating {
    self.state = GMPullToRefreshStateHidden;
}
@end



#import <objc/runtime.h>

static char UIScrollViewPullToRefreshView;

@implementation UIScrollView (GMPullToAction)

@dynamic pullToRefreshView;

- (void)addPullToRefreshWithActionHandler:(void (^)(void))actionHandler {
    GMPullToRefresh *pullToRefreshView = [[GMPullToRefresh alloc] initWithScrollView:self];
    pullToRefreshView.actionHandler = actionHandler;
    self.pullToRefreshView = pullToRefreshView;
}

- (void)setPullToRefreshView:(GMPullToRefresh *)pullToRefreshView {
    [self willChangeValueForKey:@"pullToRefreshView"];
    objc_setAssociatedObject(self, &UIScrollViewPullToRefreshView,
                             pullToRefreshView,
                             OBJC_ASSOCIATION_ASSIGN);
    [self didChangeValueForKey:@"pullToRefreshView"];
}

- (GMPullToRefresh *)pullToRefreshView {
    return objc_getAssociatedObject(self, &UIScrollViewPullToRefreshView);
}

//每次拖动都会在拖动过程中不断调用这个方法
- (void)didScroll {
    //这个self是指scrollView
    CGPoint point=self.contentOffset;
    if (self.pullToRefreshView) {
        [self.pullToRefreshView scrollViewDidScroll:point];
    }
}
@end